/*
 * ValidationHelper.java
 *
 */
package gov.nist.mu.validation;

import gov.nist.hitsp.validation.DocumentType;
import gov.nist.hitsp.validation.ValidationResults;
import gov.nist.hl7.v3.xml.util.MiscXmlUtil;

import java.io.IOException;
import java.io.InputStream;

import java.util.Iterator;
import java.util.List;

import javax.servlet.http.HttpServletRequest;
import javax.xml.parsers.DocumentBuilder;
import javax.xml.parsers.DocumentBuilderFactory;
import javax.xml.parsers.ParserConfigurationException;

import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.FileUploadException;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;
import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.w3c.dom.Document;
import org.w3c.dom.Node;
import org.xml.sax.SAXException;

/**
 *
 * @author mccaffrey
 */
public class ValidationHelper {

    List docTypes = null;
    private String errorMessage = null;
    ValidationResults validationResults = null;
    private List runnableValidations = null;
    String skeletonLocation = null;

    /**
     * Creates a new instance of ValidationHelper
     */
    public ValidationHelper() {
    }

    public void processRequest(HttpServletRequest request) {  //, boolean upload) {

        System.out.println("LOG: Processing MU Cancer Registry Report");

        this.clearTopErrorMessage();
        skeletonLocation = request.getSession().getServletContext().getInitParameter("skeletonLocation");
        try {
            if (docTypes == null) {
                String docTypesLocation = request.getSession().getServletContext().getInitParameter("docTypesLocation");
                docTypes = DocumentType.parseDocumentTypes(MiscXmlUtil.fileToDom(docTypesLocation));

            }
        } catch (ParserConfigurationException ex) {
            ex.printStackTrace();
        } catch (IOException ex) {
            ex.printStackTrace();
        } catch (SAXException ex) {
            ex.printStackTrace();
        }

    }

    public String processRequestXml(HttpServletRequest request) {

        System.out.println("LOG: Processing MU Cancer Registry Report - " + Validator.createDateOfTest() + ' ' + Validator.createTimeOfTest() + ' ' + request.getRemoteHost());

        skeletonLocation = request.getSession().getServletContext().getInitParameter("skeletonLocation");
        String schemaLocation = request.getSession().getServletContext().getInitParameter("schemaLocation");
        String ccdSchematronLocation = request.getSession().getServletContext().getInitParameter("ccdSchematronLocation");
        String cda4cdtSchematronLocation = request.getSession().getServletContext().getInitParameter("cda4cdtSchematronLocation");
        String crSchematronLocation = request.getSession().getServletContext().getInitParameter("crSchematronLocation");
        String stylesheetLocation = request.getSession().getServletContext().getInitParameter("stylesheetLocation");

        FileItemFactory factory = new DiskFileItemFactory();
        ServletFileUpload upload = new ServletFileUpload(factory);
        List items = null;

        try {
            items = upload.parseRequest(request);
        } catch (FileUploadException ex) {
            ex.printStackTrace();
            this.setTopErrorMessage("Error on document upload!");
            //   return false;
        }
        if (items == null || items.size() == 0) {
            this.setTopErrorMessage("No document found!");
        }

        Iterator it = items.iterator();
        FileItem fileItem = null;
        while (it.hasNext()) {
            FileItem fit = (FileItem) it.next();
            if ("file".equals(fit.getFieldName())) {
                fileItem = fit;
            } else if ("documentType".equals(fit.getFieldName())) {
                System.out.println(fit.getString());
            } else if ("resultDetail".equals(fit.getFieldName())) {
//                String resultDetailInput = fit.getString();
//                detailLevel = resultDetailInput;
//                this.processResultDetail(resultDetailInput);
            }
        }

        if (fileItem == null || fileItem.getSize() == 0) {
            this.setTopErrorMessage("Empty or no document found!");
        }




        SchemaValidationErrorHandler errorHandler = new SchemaValidationErrorHandler();
        Document doc = null;
        boolean fatalError = false;

        try {
            errorHandler = new SchemaValidationErrorHandler();
            doc = ValidationHelper.validateWithSchema(fileItem.getInputStream(), errorHandler, schemaLocation);
        } catch (IOException ex) {
            System.out.println("IO exception HERE!!!!! " + ex.toString());
            ex.printStackTrace();
            errorHandler.addError("Message is not valid XML.  Possible empty message.", null);
            fatalError = true;
        } catch (Exception ex) {
            System.out.println("Generic exception HERE!!!!! " + ex.toString());
            fatalError = true;
        }
        if (doc == null) {
            fatalError = true;
        }
        String ccdErrorsString = null;
        String cda4cdtErrorsString = null;
        String crErrorsString = null;

        Node ccdErrors = null;
        Node cda4cdtErrors = null;
        Node crErrors = null;
                
        if (!fatalError) {

            ccdErrorsString = Validator.validateWithSchematron(doc, ccdSchematronLocation, skeletonLocation, "errors");
            cda4cdtErrorsString = Validator.validateWithSchematron(doc, cda4cdtSchematronLocation, skeletonLocation, "errors");
            crErrorsString = Validator.validateWithSchematron(doc, crSchematronLocation, skeletonLocation, "errors");

            try {
                ccdErrors = Validator.stringToDom(ccdErrorsString);
                cda4cdtErrors = Validator.stringToDom(cda4cdtErrorsString);
                crErrors = Validator.stringToDom(crErrorsString);
            } catch (SAXException ex) {
                // TODO
            } catch (ParserConfigurationException ex) {
                // TODO
            } catch (IOException ex) {
                // TODO
            }
        }
        Node[] messageList = { ccdErrors, cda4cdtErrors, crErrors } ;
                
        Document result = null;
        if(!fatalError) 
            result = Validator.generateReport(doc, errorHandler, messageList);
        else
            result = Validator.generateReport(doc, errorHandler, null);
        
        String resultString = Validator.xmlToString(result);        
        resultString = resultString.substring(resultString.indexOf("?>") + 2);
        resultString = "<?xml version=\"1.0\" encoding=\"UTF-8\"?>\n"
                + "<?xml-stylesheet type=\"text/xsl\" href=\"" + stylesheetLocation + "\"?>"
                + resultString;

        return resultString;


    }

    public String getSchemaName() {
        if (validationResults != null && validationResults.getSchemaName() != null) {
            return validationResults.getSchemaName();
        }
        return "Schema";
    }

    public boolean hasSchemaErrors() {
        if (validationResults == null || validationResults.getSchemaErrors() == null) {
            return false;
        }
        if (validationResults.getSchemaErrors().getErrors() == null) {
            return false;
        }
        if (validationResults.getSchemaErrors().getErrors().size() == 0) {
            return false;
        }
        return true;
    }
    /*
     * public String getFileName() { return fileName; }
     */

    public List getLinesSchemaErrors() {
        return validationResults.getSchemaErrors().getLinesErrors();
    }

    public List getSchemaErrors() {
        return validationResults.getSchemaErrors().getErrors();
    }

    public int getNumberSchemaErrors() {
        return validationResults.getSchemaErrors().getNumberErrors();
    }

    public List getSchemaWarnings() {
        return validationResults.getSchemaErrors().getWarnings();
    }

    public List getLinesSchemaWarnings() {
        return validationResults.getSchemaErrors().getLinesWarnings();
    }

    public int getNumberSchemaWarnings() {
        return validationResults.getSchemaErrors().getNumberWarnings();
    }

    public boolean hasSchemaWarnings() {
        if (validationResults == null || validationResults.getSchemaErrors() == null) {
            return false;
        }
        if (validationResults.getSchemaErrors().getWarnings() == null) {
            return false;
        }
        if (validationResults.getSchemaErrors().getWarnings().size() == 0) {
            return false;
        }
        return true;
    }

    public boolean hasSchematronErrors() {
        if (validationResults.getSchematronErrors() == null) {
            return false;
        }
        if (validationResults.getSchematronErrors().size() == 0) {
            return false;
        }
        return true;
    }

    public List getSchematronErrors() {
        return validationResults.getSchematronErrors();
    }

    public boolean hasTopErrorMessage() {
        if (errorMessage == null) {
            return false;
        }
        return true;
    }

    public String getTopErrorMessage() {
        return errorMessage;
    }

    public void setTopErrorMessage(String errorMessage) {
        this.errorMessage = errorMessage;
    }

    public void clearTopErrorMessage() {
        this.errorMessage = null;
    }

    protected static Document validateWithSchema(InputStream xml, SchemaValidationErrorHandler handler, String schemaLocation) {
        DocumentBuilderFactory factory = DocumentBuilderFactory.newInstance();
        factory.setNamespaceAware(true);
        factory.setValidating(true);
        factory.setAttribute("http://java.sun.com/xml/jaxp/properties/schemaLanguage",
                "http://www.w3.org/2001/XMLSchema");

        factory.setAttribute("http://java.sun.com/xml/jaxp/properties/schemaSource",
                schemaLocation);
        factory.setIgnoringElementContentWhitespace(true);
        DocumentBuilder builder = null;
        try {
            builder = factory.newDocumentBuilder();
        } catch (ParserConfigurationException pce) {
            pce.printStackTrace();
            return null;
        }

        builder.setErrorHandler(handler);
        Document doc = null;
        try {
            doc = builder.parse(xml);
        } catch (SAXException e) {
            System.out.println("File is not valid XML. " + e.getMessage() + ' ' + e.toString());
            handler.addError("File is not valid XML.  " + e.getMessage() + ' ' + e.toString(), null);
            e.printStackTrace();
        } catch (IOException e) {
            System.out.println("File is not valid XML.  Possible empty file.");
            handler.addError("File is not valid XML.  Possible empty file.", null);
            e.printStackTrace();
        }
        return doc;
    }

    public List getRunnableValidations() {
        if (runnableValidations == null && docTypes != null) {
            runnableValidations = DocumentType.filterRunnableOnly(docTypes);
        }
        return runnableValidations;
    }

    public int getNumberOfRunnable() {
        List runnable = this.getRunnableValidations();
        if (runnable == null) {
            return 0;
        }
        return runnable.size();
    }

    public DocumentType getRunnableDocumentTypeAt(int i) {
        return (DocumentType) this.getRunnableValidations().get(i);
    }

    public int getNumberOfDependenciesAt(int i) {
        return this.getRunnableDocumentTypeAt(i).getDependencyNames(docTypes).size();
    }

    public String getDependenciesNameAt(int i, int j) {
        return (String) this.getRunnableDocumentTypeAt(i).getDependencyNames(docTypes).get(j);
    }

    public String getDependenciesIdAt(int i, int j) {
        return (String) this.getRunnableDocumentTypeAt(i).getDependencyId(docTypes).get(j);
    }

    public String getDependenciesDescriptionAt(int i, int j) {
        return (String) this.getRunnableDocumentTypeAt(i).getDependencyDescription(docTypes).get(j);
    }

    public int getIndexOfTest(String name) {
        int i = 0;
        while (i < this.getNumberOfRunnable() && !this.getRunnableDocumentTypeAt(i).getId().equals(name)) {
            i++;
        }
        if (this.getRunnableDocumentTypeAt(i).getId().equals(name)) {
            return i;
        } else {
            return -1;
        }
    }
}
